﻿using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Windows;

namespace BezierSpline.Models
{
    public class ClusterModel
    {
        const double _speedLow = 1.0;
        const double _speedHi = 1.2;
        const double _splitPoint = 0.7;
        const int _colorInterval = 300;

        private ObservableCollection<SplineModel> _lines;
        private Point[] _postion = null;
        private Point[] _vector = null;
        private double[] _r = null;
        private string _format = null;
        private string[] _availableColors = null;
        private string[] _colorAnimation = null;
        private int _currentColor;
        private double _screenWidth, _screenHeight;

        public ObservableCollection<SplineModel> Lines { get { return _lines; } }
        public ClusterModel()
        {
            _lines = new ObservableCollection<SplineModel>();
            _postion = new Point[0];
            _vector = new Point[0];
            _r = new double[0];
            _colorAnimation = new string[] { "#FFFF0000" };
        }

        public void Step(double step)
        {
            Debug.Assert(_postion.Length == _vector.Length);
            for (int i = 0; i < _postion.Length; i++)
            {
                if (i % 3 == 0)
                {
                    int j = i - 1;
                    int k = i / 3;
                    if (j < 0) j += _postion.Length;
                    _postion[i].X = (_postion[j].X - _postion[i + 1].X) * Math.Abs(_r[k] - _splitPoint) + _postion[i + 1].X;
                    _postion[i].Y = (_postion[j].Y - _postion[i + 1].Y) * Math.Abs(_r[k] - _splitPoint) + _postion[i + 1].Y;
                    _r[k] = (_r[k] + 0.003) % (_splitPoint * 2);
                }
                else
                {
                    _postion[i].X += _vector[i].X * step;
                    _postion[i].Y += _vector[i].Y * step;

                    if (_postion[i].X < 0) _vector[i].X = Math.Abs(_vector[i].X);
                    if (_postion[i].Y < 0) _vector[i].Y = Math.Abs(_vector[i].Y);
                    if (_postion[i].X > _screenWidth) _vector[i].X = -Math.Abs(_vector[i].X);
                    if (_postion[i].Y > _screenHeight) _vector[i].Y = -Math.Abs(_vector[i].Y);
                }
            }

            var en = _postion.Select(s => new object[] { s.X, s.Y }).SelectMany(s => s);

            if (_postion.Length != 0)
            {
                string data = string.Format(_format, en.ToArray());
                SplineModel line = new SplineModel();
                line.Data = data;
                if (_currentColor >= _colorAnimation.Length)
                {
                    string name = _colorAnimation.Last();
                    double A1 = int.Parse(name.Substring(1, 2), NumberStyles.HexNumber);
                    double R1 = int.Parse(name.Substring(3, 2), NumberStyles.HexNumber);
                    double G1 = int.Parse(name.Substring(5, 2), NumberStyles.HexNumber);
                    double B1 = int.Parse(name.Substring(7, 2), NumberStyles.HexNumber);
                    name = _availableColors[Tools.NextInt(_availableColors.Length)];
                    double A2 = int.Parse(name.Substring(1, 2), NumberStyles.HexNumber);
                    double R2 = int.Parse(name.Substring(3, 2), NumberStyles.HexNumber);
                    double G2 = int.Parse(name.Substring(5, 2), NumberStyles.HexNumber);
                    double B2 = int.Parse(name.Substring(7, 2), NumberStyles.HexNumber);
                    _colorAnimation = new string[_colorInterval];
                    for (int i = 0; i < _colorInterval; i++)
                    {
                        _colorAnimation[i] = string.Format("#{0:X2}{1:X2}{2:X2}{3:X2}",
                            (int)((A2 - A1) * i / _colorInterval + A1),
                            (int)((R2 - R1) * i / _colorInterval + R1),
                            (int)((G2 - G1) * i / _colorInterval + G1),
                            (int)((B2 - B1) * i / _colorInterval + B1)
                         );
                    }
                    _currentColor = 0;
                }
                line.ColorName = _colorAnimation[_currentColor++];
                _lines.Add(line);
            }
        }
        public void Fix(int ringCount, int inflexionCount)
        {
            while (_lines.Count > ringCount)
                _lines.RemoveAt(0);
            if (inflexionCount * 3 != _postion.Length)
            {
                BuildFormat(inflexionCount);
                _postion = _postion
                    .Concat(Enumerable.Range(0, Math.Max(inflexionCount * 3 - _postion.Length, 0))
                        .Select(s => new Point(Tools.NextDouble(_screenWidth), Tools.NextDouble(_screenHeight))))
                    .Take(inflexionCount * 3)
                    .ToArray();
                _vector = _vector
                    .Concat(Enumerable.Range(0, Math.Max(inflexionCount * 3 - _vector.Length, 0))
                        .Select(s => new Point(Tools.NextDouble(_speedLow, _speedHi), Tools.NextDouble(_speedLow, _speedHi))))
                    .Take(inflexionCount * 3)
                    .ToArray();
                _r = _r
                    .Concat(Enumerable.Range(0, Math.Max(inflexionCount - _r.Length, 0))
                        .Select(s => Tools.NextDouble(_splitPoint * 2)))
                    .Take(inflexionCount)
                    .ToArray();
            }
        }
        public void SetScreenSize(double screenWidth, double screenHeight)
        {
            _screenWidth = screenWidth;
            _screenHeight = screenHeight;
        }
        public void SetAvailableColors(string[] colorNames)
        {
            if (colorNames.Length == 0)
                _availableColors = new string[] { "#FFFF0000" };
            else
                _availableColors = colorNames;
        }
        private void BuildFormat(int inflexionCount)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append("M{0},{1}");
            for (int i = 0; i < inflexionCount - 1; i++)
                sb.Append(" C{" + (i * 6 + 2) + "},{" + (i * 6 + 3) + "} {" + (i * 6 + 4) + "},{" + (i * 6 + 5) + "} {" + (i * 6 + 6) + "},{" + (i * 6 + 7) + "}");
            sb.Append(" C{" + (inflexionCount * 6 - 4) + "},{" + (inflexionCount * 6 - 3) + "} {" + (inflexionCount * 6 - 2) + "},{" + (inflexionCount * 6 - 1) + "} {0},{1}");
            _format = sb.ToString();
        }
    }
}
